import pytest
from pyramid import testing
from testfixtures import ShouldRaise


class TestBaseView(object):
    @pytest.fixture
    def target(self):
        from rebecca.view import BaseView
        return BaseView

    def test_it(self, target):
        request = testing.DummyRequest()
        context = testing.DummyResource()
        result = target(context, request)

        assert result.context == context
        assert result.request == request


class TestViewSupport(object):
    @pytest.fixture
    def target(self):
        from rebecca.view import ViewSupport
        return ViewSupport

    @pytest.fixture
    def config(self, request):
        config = testing.setUp()
        def fin():
            testing.tearDown()
        request.addfinalizer(fin)
        return config

    def test_response(self, target):
        request = testing.DummyRequest()
        view_support = target()
        view_support.request = request

        assert view_support.response == request.response

    def test_redirect(self, target):
        view_support = target()
        result = view_support.redirect('http://testing.example.com')

        assert result.location == 'http://testing.example.com'

    def test_redirect_route(self, target, config):
        config.add_route('testing', 'testing/{value1}/{value2}')
        request = testing.DummyRequest()
        view_support = target()
        view_support.request = request
        result = view_support.redirect_route('testing',
                                             value1=100,
                                             value2='test')
        
        assert result.location == 'http://example.com/testing/100/test'


    def test_body(self, target):
        request = testing.DummyRequest(body='this is body')
        view_support = target()
        view_support.request = request

        assert view_support.body == 'this is body'


    def test_action_dispatch(self, target):
        request = testing.DummyRequest(
            params=dict(action='test'),
            )
        view_support = target()
        view_support.request = request
        view_support.test_action = lambda: 'OK'

        result = view_support.action_dispatch('default')
        assert result == 'OK'

    def test_action_dispatch_default(self, target):
        request = testing.DummyRequest(
            )
        view_support = target()
        view_support.request = request
        view_support.default_action = lambda: 'OK'

        result = view_support.action_dispatch('default')
        assert result == 'OK'

    def test_action_dispatch_invalid(self, target):
        request = testing.DummyRequest(
            )
        view_support = target()
        view_support.request = request

        result = view_support.action_dispatch('default')
        assert result.status_int == 400


class TestSoftification(object):

    @pytest.fixture
    def target(self):
        from rebecca.view import Softification
        return Softification

    def test_replace(self,target):
        class DummyTargetException(Exception):
            pass
        class DummyReplacedException(Exception):
            pass

        softificate = target(DummyTargetException, DummyReplacedException)

        with ShouldRaise(DummyReplacedException()):
            softificate.__exit__(DummyTargetException, DummyTargetException(), None)

    def test_reraise(self,target):
        class DummyTargetException(Exception):
            pass
        class DummyReplacedException(Exception):
            pass

        softificate = target(DummyTargetException, DummyReplacedException)

        with ShouldRaise(ImportError()):
            softificate.__exit__(ImportError, ImportError(), None)



from setuptools import setup, find_packages
import os

version = '0.2'

requires = [
    "pyramid",
    "six",
    ]

tests_require = [
    "pytest",
    "pytest-cov",
    "testfixtures",
    ]

long_description = (
    open('README.txt').read()
    + '\n' +
    'Contributors\n'
    '============\n'
    + '\n' +
    open('CONTRIBUTORS.txt').read()
    + '\n' +
    open('CHANGES.txt').read()
    + '\n')

setup(name='rebecca.view',
      version=version,
      description="basic views for pyramid view",
      long_description=long_description,
      # Get more strings from
      # http://pypi.python.org/pypi?%3Aaction=list_classifiers
      classifiers=[
        "Programming Language :: Python",
        "Programming Language :: Python :: 2.6",
        "Programming Language :: Python :: 2.7",
        "Programming Language :: Python :: 3",
        "Programming Language :: Python :: 3.2",
        "Programming Language :: Python :: 3.3",
        "Programming Language :: Python :: Implementation :: CPython",
        "Programming Language :: Python :: Implementation :: PyPy",
        "Framework :: Pyramid",
        "License :: OSI Approved :: MIT License",
        ],
      keywords='',
      author='Atsushi Odagiri',
      author_email='aodagx@gmail.com',
      url='https://github.com/rebeccaframework/rebecca.view',
      license='MIT',
      packages=find_packages('src'),
      package_dir = {'': 'src'},
      namespace_packages=['rebecca'],
      include_package_data=True,
      zip_safe=False,
      install_requires=requires,
      tests_require=tests_require,
      extras_require={
        "testing": tests_require,
        },
      entry_points="""
      # -*- Entry points: -*-
      """,
      )



from pyramid.httpexceptions import HTTPFound, HTTPBadRequest
from six import reraise

class Softification(object):
    def __init__(self, targets, replacer):
        self.targets = targets
        self.replacer = replacer

    def __enter__(self):
        pass

    def __exit__(self, exc_type, exc_val, exc_tb):
        if exc_val is None:
            return

        if isinstance(exc_val, self.targets):
            raise self.replacer()

        reraise(exc_type, exc_val, exc_tb)


class BaseView(object):
    def __init__(self, context, request):
        self.context = context
        self.request = request


class ViewSupport(object):
    @property
    def response(self):
        return self.request.response

    def redirect(self, url):
        return HTTPFound(location=url)

    def redirect_route(self, route_name, **values):
        return self.redirect(self.request.route_url(route_name, **values))

    @property
    def body(self):
        return self.request.body

    def action_dispatch(self, default):
        action = self.request.params.get('action', default)
        mname = action + '_action'
        if not hasattr(self, mname):
            return HTTPBadRequest()
        return getattr(self, mname)()


class BasicView(BaseView, ViewSupport):
    pass

class ActionView(BasicView):
    def __call__(self):
        return self.action_dispatch()
